@@ -5,23 +5,26 @@
 #'
 #' @param df \code{data.frame}. One or more datasets appended together.
 #' @param sn Unique \code{numeric} record identifier. Optional.
-#' @param strata Column names. Episodes will be unique to each \code{strata}. \code{\link{record_group}} can be used to create \code{strata} within datasets.
-#' @param date Record date or interval. \code{date}, \code{datetime} or \code{\link{number_line}} objects.
-#' @param case_length Period from a \code{"Case"} within which another record of the same \code{strata} is considered a \code{"Duplicate"} record.
+#' @param strata Episode grouping will be done separately for these subsets (\code{strata}) of the dataset. \code{episode_group} supports the use of multiple columns supplied as column names. \code{\link{record_group}} can be used to create \code{strata}.
+#' @param date Record date (\code{date} or \code{datetime}) or period (\code{\link{number_line}}) objects.
+#' @param case_length Period after a \code{"Case (C)"} within which another record from the same \code{strata} is considered a \code{"Duplicate (D)"} record.
 #' @param episodes_max Maximum number of times to group episodes within each \code{strata}.
 #' @param episode_type \code{"fixed"} or \code{"rolling"}.
-#' @param recurrence_length Period from the last record of an episode within which another record of the same \code{strata} is considered a \code{"Recurrent"} record. If a \code{recurrence_length} is not supplied, the \code{case_length} is used.
+#' @param recurrence_length Period after the last record (\code{"Case (C)"} , \code{"Duplicate (D)"} or \code{"Recurrent (R)"}) in an episode within which another record from the same \code{strata} is considered a \code{"Recurrent (R)"} record. If \code{recurrence_length} is not supplied, \code{case_length} is used as \code{recurrence_length}.
 #' @param episode_unit Time units as supported by lubridate's \code{\link[lubridate]{duration}} function.
-#' @param rolls_max Maximum number of recurrence permitted within each episode. Only used if \code{episode_type} is \code{"rolling"}.
-#' @param data_source Unique dataset identifier for the \code{data.frame}. Useful when \code{data.frame} contains multiple datasets.
+#' @param rolls_max Maximum number of times an event an reoccur within an episode. Only used if \code{episode_type} is \code{"rolling"}.
+#' @param data_source Unique dataset identifier. Useful when the dataset contains data from multiple sources. \code{episode_group} support the use of multiple columns supplied as column names.
 #' @param from_last If \code{TRUE}, episode grouping will be backwards in time - starting at the most recent record and proceeding to the earliest. If \code{FALSE}, it'll be forward in time - starting at the earliest record and proceeding to the most recent one.
-#' @param overlap_method A set of methods for grouped intervals to overlap. Options are; \code{"across"}, \code{"aligns_start"}, \code{"aligns_end"}, \code{"inbetween"}, \code{"chain"}. See \code{\link{overlap}} functions.
-#' @param custom_sort If \code{TRUE}, \code{"Case"} assignment will be in preference to this sort order. Useful in specifying that episode grouping begins at a particular kind of record regardless of chronological order.
-#' @param bi_direction If \code{FALSE}, \code{"Duplicate"} records will be those within the \code{case_length} and \code{recurrence_length}, before or after the \code{"Case"} as determined by \code{from_last}. If \code{TRUE}, \code{"Duplicate"} records will be those on both sides of the \code{"Case"}.
+#' @param overlap_method A set of ways for grouped intervals to overlap. Options are; \code{"across"}, \code{"aligns_start"}, \code{"aligns_end"}, \code{"inbetween"}, \code{"chain"}. See \code{\link{overlap}} functions.
+#' @param custom_sort If \code{TRUE}, \code{"Case (C)"} assignment will be in preference to this sort order. Useful in specifying that episode grouping begins at particular records regardless of chronological order. \code{episode_group} supports the use of multiple columns supplied as column names.
+#' @param bi_direction If \code{FALSE}, \code{"Duplicate (D)"} records will be those within the \code{case_length} period, before or after the \code{"Case (C)"} as determined by \code{from_last}. If \code{TRUE}, \code{"Duplicate (D)"} records will be those within the same period before and after the \code{"Case (C)"}.
 #' @param group_stats If \code{TRUE}, the output will include additional columns with useful stats for each episode group.
 #' @param display If \code{TRUE}, status messages are printed on screen.
+#' @param to_s4 if \code{TRUE}, changes the returned value to an \code{\link[=epid-class]{epid}} object.
 #'
-#' @return \code{episode_group} - \code{data.frame}
+#' @return
+#'
+#' \code{data.frame} (\code{\link[=epid-class]{epid}} objects if \code{to_s4} is \code{TRUE})
 #'
 #' \itemize{
 #' \item \code{sn} - unique record identifier as provided
@@ -29,107 +32,148 @@
 #' \item \code{case_nm} - record type in regards to case assignment
 #' \item \code{epid_dataset} - datasets contained in each episode
 #' \item \code{epid_interval} - Episode start and end dates. \code{\link{number_line}} object.
-#' \item \code{epid_length} - Difference between episode start and end dates. \code{difftime} object. If possible, the same unit supplied to \code{episode_unit} is used otherwise, a difference in days is returned
-#' \item \code{epid_total} - number of records in each record group
+#' \item \code{epid_length} - Difference between episode start and end dates (\code{difftime}). If possible, it's the same unit as \code{episode_unit} otherwise, a difference in days is returned
+#' \item \code{epid_total} - number of records in each episode
 #' }
 #'
+#' \code{epid} objects will be the default output from the next release onwards
+#'
 #' @seealso
 #' \code{\link{record_group}}, \code{\link{overlap}} and \code{\link{number_line}}
 #'
 #' @details
-#' Episode grouping begins at a reference record (\code{"Case"}) and proceeds forward or backward in time depending on \code{from_last}.
-#' If \code{custom_sort} is used, episode grouping can be forced to begin at certain record before proceeding forward or backwards in time.
+#' Episode grouping begins at a reference record (\code{"Case (C)"}) and proceeds forward or backward in time depending on \code{from_last}.
+#' If \code{custom_sort} is used, episode grouping can be forced to begin at certain records before proceeding forward or backwards in time.
 #' The maximum duration of a \code{"fixed"} episode is the \code{case_length} while, the maximum duration of a \code{"rolling"} episode is the
-#' \code{case_length} in addition to all recurrence periods. A recurrence period is the \code{recurrence_length} from the last record in an episode
+#' \code{case_length} plus all recurrence periods. A recurrence period is a fixed period (\code{recurrence_length}) after the last record in an episode.
 #'
 #' @examples
 #' library(dplyr)
 #' library(lubridate)
-#' library(diyar)
 #'
+#' #1. Fixed episodes
 #' data(infections); infections
-#'
+#' db_1 <- infections
+#' # 16-day (difference of 15 days) episodes beginning from the earliest record
+#' db_1$fd <- fixed_episodes(db_1$date, case_length = 15, to_s4 = TRUE, display = FALSE)
 #' # 16-hour (difference of 15 hours) episodes beginning from the earliest record
-#' epids <- episode_group(infections, sn = rd_id, date = date, case_length = epi_len,
-#' from_last = TRUE, episode_unit = "hours", group_stats = TRUE)
-#' left_join(infections, epids, by=c("rd_id"="sn"))
+#' db_1$fh <- fixed_episodes(db_1$date, case_length = 15,
+#' episode_unit = "hours", to_s4 = TRUE, display = FALSE)
+#' db_1
 #'
-#' # One rolling episode per strata. Initial case_length of 16 days (difference of 15 days) and
-#' # one recurrence period of 31 days (difference of 30 days)
-#' infections$recur <- 30
-#' epids <- episode_group(infections, date=date, case_length = epi_len, episode_type = "rolling",
-#' recurrence_length = recur, episodes_max = 1, rolls_max = 1, display = FALSE, group_stats = TRUE)
-#' bind_cols(infections, epids)
+#' #2. Rolling episodes
+#' # Case length and recurrence periods of 15 days
+#' db_1$rd_a <- rolling_episodes(db_1$date, case_length = 15, to_s4 = TRUE, display = FALSE)
+#' # Case length of 15 days and recurrence periods of 10 days
+#' db_1$rd_b <- rolling_episodes(db_1$date, case_length = 15,
+#' recurrence_length = 10, to_s4 = TRUE, display = FALSE)
+#' # Case length of 15 days and 2 recurrence periods of 10 days
+#' db_1$rd_c <- rolling_episodes(db_1$date, case_length = 15,
+#' recurrence_length = 10, rolls_max = 2, to_s4 = TRUE, display = FALSE)
+#' db_1
 #'
-#' # User defined case assignment
-#' # Preference for case assignment - UTI > BSI > RTI
-#' infections$infx <- factor(infections$infection, levels = c("UTI","BSI","RTI"))
+#' # 3. Stratified episode grouping
+#' db_3 <- infections
 #'
-#' # Different case and recurrence lengths for different sources of infection
-#' infections <- mutate(infections,
-#'                      epi_len = case_when(
-#'                        infection == "BSI" ~ 14,
-#'                        infection == "UTI" ~ 30,
-#'                        infection == "RTI" ~ 60
-#'                      )
-#' )
+#' db_3$patient_id <- c(rep("PID 1",8), rep("PID 2",3))
+#' # One 16-day episode per patient
+#' db_3$epids_p <- fixed_episodes(date=db_3$date, strata = db_3$patient_id,
+#' case_length = 15, episodes_max = 1, to_s4 = TRUE, display = FALSE)
+#' db_3
 #'
-#' # n-day episodes beginning with the earliest record with the specified preference; UTI > BSI > RTI
-#' epids <- episode_group(infections, rd_id, date=date, case_length = epi_len,
-#'                        custom_sort = infx, group_stats = TRUE,  display = FALSE)
-#' bind_cols(infections, epids)
+#' # 4. Case assignment
+#' db_4 <- infections
 #'
-#' # Another preference - RTI > UTI, or  RTI > BSI, or earliest record
-#' infections$infx <- ifelse(infections$infection =="RTI",0,1)
-#' epids <- episode_group(infections, rd_id, date=date, case_length = epi_len,
-#' custom_sort = infx, from_last = TRUE, bi_direction = TRUE, display = FALSE, group_stats = TRUE)
-#' bind_cols(infections, epids)
+#' ## 4.1 Chronological order
+#' db_4$forward_time <- fixed_episodes(db_4$date, case_length = 1,
+#' episode_unit = "month", to_s4 = TRUE, display = FALSE)
+#' db_4$backward_time <- fixed_episodes(db_4$date, case_length = 1,
+#' episode_unit = "month", from_last = TRUE, to_s4 = TRUE, display = FALSE)
+#' db_4
 #'
-#' # Stratified episode grouping
-#' infections$patient_id <- c(rep("PID 1",8), rep("PID 2",3))
+#' ## 4.2 User defined order
+#' db_4b <- infections
+#' # RTI > UTI, or RTI > BSI
+#' db_4b$ord2 <- ifelse(db_4b$infection =="RTI",0,1)
+#' # UTI > BSI > RTI
+#' db_4b$ord1 <- factor(db_4b$infection, levels = c("UTI","BSI","RTI"))
 #'
-#' # Only three 9-day (difference of 8 days) rolling episodes per patient and infection.
-#' infections$epi_len <- 8
-#' epids <- episode_group(infections, rd_id, date=date, strata = c(patient_id, infection),
-#' case_length = epi_len, episode_type = "rolling", recurrence_length = recur, episodes_max = 3,
-#' data_source = c(patient_id, infection), display = FALSE)
+#' db_4b$epids_1 <- fixed_episodes(db_4b$date, case_length = 15,
+#' custom_sort = db_4b$ord2, to_s4 = TRUE, display = FALSE)
+#' db_4b$epids_2 <- fixed_episodes(db_4b$date, case_length = 15,
+#' custom_sort = db_4b$ord1, to_s4 = TRUE, display = FALSE)
+#' db_4b$epids_2b <- fixed_episodes(db_4b$date, case_length = 15,
+#' custom_sort = db_4b$ord1, bi_direction = TRUE, to_s4 = TRUE, display = FALSE)
+#' db_4b
 #'
-#' bind_cols(infections, epids)
-#'
-#' # Interval grouping
-#' data(hospital_admissions); hospital_admissions
+#' #5. Interval grouping
+#' data(hospital_admissions)
 #'
 #' hospital_admissions$admin_period <- number_line(hospital_admissions$admin_dt,
 #' hospital_admissions$discharge_dt)
-#' hospital_admissions <- select(hospital_admissions, -c(discharge_dt, admin_dt))
+#' admissions <- hospital_admissions[c("admin_period","epi_len")]
+#' admissions
 #'
-#' # Episodes of overlaping intervals of admission
-#' epids <- episode_group(hospital_admissions, date=admin_period,
-#' sn=rd_id, case_length = epi_len, group_stats = TRUE)
-#' bind_cols(hospital_admissions, epids)
+#' # Episodes of overlaping periods of admission
+#' admissions$epi_0 <- fixed_episodes(date=admissions$admin_period, case_length = 0,
+#' group_stats = TRUE, to_s4=TRUE)
+#' admissions
 #'
-#' # Overlaping intervals of admission seperated by 1 month
-#' hospital_admissions$epi_len <- 1
-#' epids <- episode_group(hospital_admissions, date=admin_period, sn=rd_id,
-#' case_length = epi_len, episode_unit = "months")
-#' bind_cols(hospital_admissions, epids)
+#' # Overlaping periods of admission seperated by 1 month
+#' admissions$epi_1 <- fixed_episodes(date=admissions$admin_period, case_length = 1,
+#' episode_unit = "months", group_stats = TRUE, to_s4 = TRUE, display = FALSE)
+#' admissions
 #'
-#' # Episodes of chained intervals, and those with aligned end periods
-#' hospital_admissions$epi_len <- 0
-#' epids <- episode_group(hospital_admissions, date=admin_period, sn=rd_id,
-#' case_length = epi_len, overlap_method = c("chain","aligns_end"))
-#' bind_cols(hospital_admissions, epids)
+#' # Episodes of chained admission periods, and those with aligned end periods
+#' admissions$epi_0b <- fixed_episodes(date=admissions$admin_period, case_length = 0,
+#' overlap_method = c("chain","aligns_end"), group_stats = TRUE, to_s4 = TRUE, display = FALSE)
+#' admissions["epi_0b"]
+#'
+#'
+#' # Note - episode_group() takes column names not actual values
+#' db_5 <- infections
+#'
+#' db_5$recur <- 20
+#' db_5$epids_f <- episode_group(db_5, date=date, episode_type = "fixed",
+#' case_length = epi_len, to_s4 = TRUE, display = FALSE)
+#' db_5$epids_r <- episode_group(db_5, date=date, episode_type = "rolling",
+#' case_length = epi_len, recurrence_length = recur, to_s4 = TRUE, display = FALSE)
+#' db_5
 #'
 #' @importFrom dplyr %>%
 #' @importFrom rlang .data
-
+#'
 #' @aliases episode_group
 #' @export
 episode_group <- function(df, sn = NULL, strata = NULL, date,
                           case_length, episode_type="fixed", episode_unit = "days", episodes_max = Inf,
                           recurrence_length = NULL, rolls_max =Inf, data_source = NULL,
-                          custom_sort = NULL, from_last=FALSE, overlap_method = c("across","inbetween","aligns_start","aligns_end","chain"), bi_direction = FALSE, group_stats= FALSE, display=TRUE){
+                          custom_sort = NULL, from_last=FALSE, overlap_method = c("across","inbetween","aligns_start","aligns_end","chain"), bi_direction = FALSE,
+                          group_stats= FALSE, display=TRUE, deduplicate=FALSE, to_s4 = FALSE){
   . <- NULL
+  if(!(is.logical(group_stats) & is.logical(from_last) & is.logical(display) & is.logical(to_s4))) stop(paste("'group_stats', 'from_last', 'display' and 'to_s4' must be TRUE or FALSE"))
+  if(to_s4 == FALSE){
+    # check if episode_group() was called by fixed_episodes() or rolling_episodes()
+    wrap_func <- c("rolling_episodes","fixed_episodes")
+    call <- deparse(sys.call(-(sys.nframe()-1)))
+    lg <- unlist(lapply(wrap_func, function(x){
+      grepl(paste("^",x,"\\(",sep=""), call)
+    }))
+
+    # if not, display the message
+    if(all(!lg)){
+      if (is.null(getOption("diyar.episode_group.output"))){
+        options("diyar.episode_group.output"= TRUE)
+      }
+      if (getOption("diyar.episode_group.output")){
+        message(paste("The default output of episode_group() will be changed to epid objects in the next release.",
+                      "Please consider switching earlier by using 'to_s4=TRUE' or to_s4()",
+                      "This message is displayed once per session.", sep = "\n"))
+      }
+      options("diyar.episode_group.output"= FALSE)
+    }
+  }
+
   episodes_max <- ifelse(is.numeric(episodes_max) & !is.na(episodes_max) & !is.infinite(episodes_max), as.integer(episodes_max), episodes_max)
   rolls_max <- ifelse(is.numeric(rolls_max) & !is.na(rolls_max) & !is.infinite(rolls_max), as.integer(rolls_max), rolls_max)
 
@@ -144,26 +188,28 @@ episode_group <- function(df, sn = NULL, strata = NULL, date,
 
   if(!episode_type %in% c("rolling","fixed") ) stop(paste("`episode_type` must be either 'rolling' or 'fixed'"))
 
-  enq_vr <- function(x, vr){
-    x <- names(dplyr::select(x, !!vr))
-
-    if(length(x)==0){
-      x <- NULL
-    }else{
-      x
-    }
-    return(x)
+  enq_vr <- function(x){
+    x <- as.character(x)
+    if(x[1]=="c" & length(x)>1) x <- x[2:length(x)]
+    if(length(x)==0) x <- NULL
+    x
   }
 
   fmt <- function(g) formatC(g, format="d", big.mark=",")
 
-  rd_sn <- enq_vr(df, dplyr::enquo(sn))
-  ds <- enq_vr(df, dplyr::enquo(data_source))
-  epl <- enq_vr(df, dplyr::enquo(case_length))
-  r_epl <- enq_vr(df, dplyr::enquo(recurrence_length))
-  st <- enq_vr(df, dplyr::enquo(strata))
-  ref_sort <- enq_vr(df, dplyr::enquo(custom_sort))
-  dt <- enq_vr(df, dplyr::enquo(date))
+  rd_sn <- enq_vr(substitute(sn))
+  ds <- enq_vr(rlang::enexpr(data_source))
+  epl <- enq_vr(substitute(case_length))
+  r_epl <- enq_vr(substitute(recurrence_length))
+  st <- enq_vr(substitute(strata))
+  ref_sort <- enq_vr(substitute(custom_sort))
+  dt <- enq_vr(substitute(date))
+
+  if(any(!unique(c(rd_sn, ds, epl, r_epl, st, ref_sort, dt)) %in% names(df))){
+    missing_cols <- subset(unique(c(rd_sn, ds, epl, r_epl, st, ref_sort, dt)), !unique(c(rd_sn, ds, epl, r_epl, st, ref_sort, dt)) %in% names(df))
+    missing_cols <- paste(paste("'",missing_cols,"'",sep=""), collapse = "," )
+    stop(paste(missing_cols, "not found"))
+  }
 
   if(!is.null(rd_sn)){
     if(!(all(df[[rd_sn]] > 0) & is.numeric(as.numeric(df[[rd_sn]])))) stop(paste("'",rd_sn,"' as 'sn' must be > 0", sep=""))
@@ -216,7 +262,8 @@ episode_group <- function(df, sn = NULL, strata = NULL, date,
 
   df <- df %>%
     dplyr::select(.data$sn, .data$rec_dt_ai, .data$rec_dt_zi, .data$epi_len, .data$rc_len, .data$dsvr, .data$cri, !!dplyr::enquo(custom_sort)) %>%
-    dplyr::mutate(tag = 0, epid = 0, case_nm="", pr_sn = dplyr::row_number(), roll=0, episodes=0)
+    dplyr::mutate(tag = 0, epid = 0, case_nm="", pr_sn = dplyr::row_number(), roll=0, episodes=0) %>%
+    dplyr::mutate_at(c("rec_dt_ai", "rec_dt_zi"), ~ lubridate::dmy_hms(format(., "%d/%m/%Y %H:%M:%S")))
 
   if(from_last==TRUE){
     df$ord <- abs(max(df$rec_dt_ai) - df$rec_dt_ai)
@@ -231,7 +278,7 @@ episode_group <- function(df, sn = NULL, strata = NULL, date,
   }else{
     ord_ls <- dplyr::select(df, ref_sort) %>%
       dplyr::mutate_all(list(~as.numeric(as.factor(.)))) %>%
-      dplyr::mutate_all(list(~stringr::str_pad(., width = max(stringr::str_length(.)), pad=0) )) %>%
+      dplyr::mutate_all(list(~formatC(., width = max(nchar(.)), flag=0))) %>%
       tidyr::unite("ord")
 
     df$user_ord <- ord_ls[[1]]
@@ -274,17 +321,13 @@ episode_group <- function(df, sn = NULL, strata = NULL, date,
     df$tr_c_int <- suppressWarnings(diyar::number_line(df$tr_rec_dt_ai, df$tr_rec_dt_zi))
     df$tr_r_int <- suppressWarnings(diyar::number_line(df$tr_rec_dt_ai, df$tr_rec_dt_zi))
 
+    bdir <- ifelse(bi_direction,"both","end")
     if (from_last==FALSE){
-      df$tr_c_int <-  suppressWarnings(diyar::expand_number_line(df$tr_c_int, df$tr_epi_len, "end"))
-      df$tr_r_int <-  suppressWarnings(diyar::expand_number_line(df$tr_r_int, df$tr_rc_len, "end"))
+      df$tr_c_int <-  suppressWarnings(diyar::expand_number_line(df$tr_c_int, df$tr_epi_len, bdir))
+      df$tr_r_int <-  suppressWarnings(diyar::expand_number_line(df$tr_r_int, df$tr_rc_len, bdir))
     }else{
-      df$tr_c_int <-  suppressWarnings(diyar::expand_number_line(df$tr_c_int, -df$tr_epi_len, "end"))
-      df$tr_r_int <-  suppressWarnings(diyar::expand_number_line(df$tr_r_int, -df$tr_rc_len, "end"))
-    }
-
-    if (bi_direction==TRUE){
-      df$tr_c_int <-  suppressWarnings(diyar::expand_number_line(df$tr_c_int, df$tr_epi_len, "both"))
-      df$tr_r_int <-  suppressWarnings(diyar::expand_number_line(df$tr_r_int, df$tr_rc_len, "both"))
+      df$tr_c_int <-  suppressWarnings(diyar::expand_number_line(df$tr_c_int, -df$tr_epi_len, bdir))
+      df$tr_r_int <-  suppressWarnings(diyar::expand_number_line(df$tr_r_int, -df$tr_rc_len, bdir))
     }
 
     df$r_range <- df$c_range <- FALSE
@@ -301,7 +344,6 @@ episode_group <- function(df, sn = NULL, strata = NULL, date,
       df$r_range <- ifelse(df$tr_rec_dt_ai < df$rec_dt_ai, FALSE, df$r_range)
     }
 
-
     df <- df %>%
       dplyr::mutate(
         epid_type = ifelse(
@@ -374,6 +416,8 @@ episode_group <- function(df, sn = NULL, strata = NULL, date,
       epid= ifelse(.data$epid==0, .data$sn, .data$epid)
     )
 
+  if(deduplicate) df <- subset(df, df$case_nm!="Duplicate")
+
   if(is.null(ds)){
     df <- dplyr::select(df, .data$sn, .data$epid, .data$case_nm, .data$pr_sn, .data$rec_dt_ai, .data$rec_dt_zi, .data$ord, .data$ord_z, .data$epi_len)
     df <- dplyr::arrange(df, .data$pr_sn)
@@ -387,7 +431,7 @@ episode_group <- function(df, sn = NULL, strata = NULL, date,
       dplyr::arrange(.data$dsvr) %>%
       tidyr::spread(key="dsvr", value="val") %>%
       tidyr::unite("epid_dataset", sourc_list, sep=",") %>%
-      dplyr::mutate(epid_dataset = stringr::str_replace_all(.data$epid_dataset,"NA,|,NA|^NA$","")) %>%
+      dplyr::mutate(epid_dataset = gsub("NA,|,NA|^NA$","",.data$epid_dataset)) %>%
       dplyr::full_join(df, by="epid")
 
     df <- dplyr::arrange(df, .data$pr_sn)
@@ -412,7 +456,7 @@ episode_group <- function(df, sn = NULL, strata = NULL, date,
 
     df <- dplyr::left_join(df, epid_l, by="epid")
 
-    diff_unit <- stringr::str_replace(tolower(episode_unit), "s$","")
+    diff_unit <- gsub("s$","",tolower(episode_unit))
     diff_unit <- ifelse(!diff_unit %in% c("second","minute","hour","day"), "day", diff_unit)
 
     df <- dplyr::mutate(df, epid_length = lubridate::make_difftime(difftime(lubridate::dmy_hms(.data$z), lubridate::dmy_hms(.data$a), units = "secs"), units = diff_unit))
@@ -433,220 +477,149 @@ episode_group <- function(df, sn = NULL, strata = NULL, date,
 
   pd <- ifelse(display,"\n","")
   cat(paste(pd, "Episode grouping complete - " ,fmt(unique_ids)," record(s) assinged a unique ID. \n" , sep =""))
+  if(to_s4) df <- diyar::to_s4(df)
   df
 }
 
-
 #' @rdname episode_group
-#' @param x Record date or interval. \code{date}, \code{datetime}, \code{number_line} objects or other \code{numeric} based objects.
-#' @param deduplicate if \code{TRUE}, retains only one the \code{"Case"} from an episode group.
-#'
+#' @param deduplicate if \code{TRUE}, retains only one the \code{"Case"} (C) record in an episode.
+#' @param x Record date or interval. Deprecated. Please use \code{date}
 #' @details
-#' \code{fixed_episodes} and \code{rolling_episodes} are more convenient implementations of \code{episode_group}.
-#' However, these are less efficient in dealing with large datasets, and lack the following features;
-#' \code{"custom_sort", "rolls_max", "episodes_max", "data_source", "episode_unit", "bi_direction" and "group_stats"}
-#'
-#' @return \code{fixed_episodes} and \code{rolling_episodes} - \code{number_line}.
-#' \itemize{
-#' \item \code{id} - unique record identifier as provided
-#' \item \code{gid} - unique episode identifier
-#' \item \code{start} - Episode start dates
-#' \item \code{.Data} - Difference between episode start and end dates. \code{numeric} object
-#' }
-#'
-#' Use \code{\link{number_line_width}} to extract the \code{epid_interval}
-#'
-#' Use \code{\link{right_point}} or \code{\link{end_point}} to extract the episode end date
-#'
-#' @examples
-#' # Convenient versions of episode_group
-#' # Episodes from time points
-#' dts <- c("13/04/2019", "01/04/2019", "05/05/2019", "10/04/2019", "01/05/2019")
-#' dts <- as.Date(dts, "%d/%m/%Y")
-#' dts
-#'
-#' epids <- fixed_episodes(dts, case_length = 5, display = FALSE)
-#' epids; str(epids); unique(epids)
-#'
-#' # Episodes from time periods
-#' pds <- as.number_line(dts)
-#' pds <- expand_number_line(pds, 1, "end")
-#' pds
-#'
-#' epids <- rolling_episodes(pds, case_length = 5, recurrence_length =11,
-#' deduplicate = TRUE, display = FALSE)
-#' epids; str(epids)
-#'
-#' db_a <- infections
-#' db_b <- mutate(db_a, epid_interval= fixed_episodes(x = date, case_length = epi_len,
-#' strata = infection, from_last = FALSE, display = FALSE, deduplicate = FALSE))
-#'
-#' db_b
-#' str(db_b$epid_interval)
-#' db_b$epid <- db_b$epid_interval$gid
-#' db_b
+#' \code{fixed_episodes()} and \code{rolling_episodes()} are wrapper functions of \code{episode_group}.
+#' They can be more convenient options and have the same functionalities as \code{episode_group()}.
 #'
 #' @export
-fixed_episodes <- function(x, strata = NULL, case_length, episodes_max = Inf, from_last = FALSE, overlap_method = c("across","inbetween","aligns_start","aligns_end","chain"), deduplicate = FALSE, display = TRUE){
+fixed_episodes <- function(date, sn = NULL, strata = NULL, case_length, episode_unit = "days", episodes_max = Inf, data_source = NULL, custom_sort = NULL,
+                           from_last = FALSE, overlap_method = c("across","inbetween","aligns_start","aligns_end","chain"),
+                           bi_direction= FALSE, group_stats = FALSE, display = TRUE, deduplicate = FALSE, x, to_s4 = FALSE){
+  if(to_s4 == FALSE){
+    if (is.null(getOption("diyar.fixed_episodes.output"))){
+      options("diyar.fixed_episodes.output"= TRUE)
+    }
+    if (getOption("diyar.fixed_episodes.output")){
+      message(paste("The default output of fixed_episodes() will be changed to epid objects in the next release.",
+                    "Please consider switching earlier by using 'to_s4=TRUE' or to_s4()",
+                    "This message is displayed once per session.", sep = "\n"))
+    }
+    options("diyar.fixed_episodes.output"= FALSE)
+  }
+  if (!missing(x)) {
+    warning("'x' is deprecated; please use 'date' instead."); date <- x
+    }
 
   if(!is.character(overlap_method)) stop(paste("'overlap_method' must be a character object"))
   if(all(!tolower(overlap_method) %in% c("across","chain","aligns_start","aligns_end","inbetween"))) stop(paste("`overlap_method` must be either 'across','chain','aligns_start','aligns_end' or 'inbetween'"))
   if(!(is.logical(from_last) & is.logical(display) & is.logical(deduplicate) )) stop(paste("'from_last', 'deduplicate' and 'display' must be TRUE or FALSE"))
   if(!all(is.finite(case_length) & case_length >= 0) ) stop(paste("'case_length' must be a numeric based object of length 1",sep=""))
-  if(!all(is.finite(x))) stop(paste("All 'x' values must be a date, datetime, number_line object or a numeric based object",sep=""))
-  if(!(length(case_length) %in% c(1, length(x)))) stop(paste("length of 'case_length' must be 1 or the same as 'x'",sep=""))
-  if(!(length(strata) %in% c(1, length(x)) | (length(strata) ==0 & is.null(strata)))) stop(paste("length of 'strata' must be 1 or the same as 'x'",sep=""))
+  if(!all(is.finite(date))) stop(paste("All 'date' values must be a date, datetime, number_line object or a numeric based object",sep=""))
+  if(!(length(case_length) %in% c(1, length(date)))) stop(paste("length of 'case_length' must be 1 or the same as 'date'",sep=""))
+  if(!(length(strata) %in% c(1, length(date)) | (length(strata) ==0 & is.null(strata)))) stop(paste("length of 'strata' must be 1 or the same as 'date'",sep=""))
+  if(!(length(data_source) %in% c(1, length(date)) | (length(data_source) ==0 & is.null(data_source)))) stop(paste("length of 'data_source' must be 1 or the same as 'date'",sep=""))
+  if(!(length(custom_sort) %in% c(1, length(date)) | (length(custom_sort) ==0 & is.null(custom_sort)))) stop(paste("length of 'custom_sort' must be 1 or the same as 'date'",sep=""))
 
-  fmt <- function(g) formatC(g, format="d", big.mark=",")
-  if(any(!diyar::is.number_line(x))){
-    x  <- diyar::as.number_line(x)
-  }
-
-  x@gid <- 1:length(x)
+  df <- data.frame(dts = date, epl = case_length, stringsAsFactors = FALSE)
 
   if(is.null(strata)){
-    x <- sort.number_line(x, decreasing = from_last)
-    s <- rep(1, length(x))
+    df$sr <- 1
   }else{
-    x <- data.frame(id= x@id, s = strata, l = x@start, nl = x)
-    db <- x[order(x$s, x$l, x$id, decreasing = from_last),]
-    x <- db$nl
-    s <- as.numeric(as.factor(db$s))
-    rm(db)
+    df$sr <- strata
   }
 
-  pr_sn <- diyar::number_line(0,0, gid =x@gid, id =x@id)
-
-  if(any(duplicated(x@id) | is.na(x@id))) stop(paste("'id' slot of the number_line object must be contain unique finite numbers",sep=""))
-
-  x@gid <- x@id
-  x <- diyar::reverse_number_line(x, "decreasing")
-
-  j <- 0
-  t <- rep(0, length(x))
-  pt <- ifelse(from_last,"start","end")
-  if(length(case_length)==1) case_length <- rep(case_length, length(x@id))
-  while (min(t) ==0 & j!=length(x)){
-    total_1 <- length(t[t==0])
-    if(display){cat(paste("Episode window ",j+1,".\n", sep=""))}
-    l <- x[t==0][1]
-    l_l <- case_length[t==0][1]
-    l_s <- s[t==0][1]
-    h <- (x@id == l@id | (diyar::overlap(diyar::expand_number_line(l, l_l, pt), x, method = overlap_method) & s ==l_s)) & t != 1
-    x[which(h)]@.Data <- as.numeric(max(diyar::right_point(x[which(h),]))) - as.numeric(min(x[which(h),]@start))
-    x[which(h)]@start <- min(x[which(h),]@start)
-    x[which(h)]@gid <- l@gid
-
-    tagged_1 <- length(h[h])
-    t[which(h)] <- 1
-    if(display){
-      cat(paste(fmt(tagged_1)," of ", fmt(total_1)," record(s) grouped into episodes. ", fmt(total_1-tagged_1)," records not yet grouped.\n", sep =""))
-    }
-    if(min(t)==1) break
-    j <- j + 1
+  if(is.null(sn)){
+    df$sn <- 1:nrow(df)
+  }else{
+    df$sn <- sn
   }
 
-  x@id <- pr_sn@id
-  db <- data.frame(nl=x, sn=pr_sn@gid)
-  x <- db[order(db$sn),]$nl
-  rm(db)
+  if(is.null(data_source)){
+    df$ds <- 1
+    ds <- NULL
+  }else{
+    df$ds <- data_source
+    ds <- "ds"
+  }
 
-  if(deduplicate) x <- unique.number_line(x)
-  if(from_last) x <- diyar::reverse_number_line(x)
-  return(x)
+  if(is.null(custom_sort)){
+    df$user_srt <- 0
+  }else{
+    df$user_srt <- as.numeric(as.factor(custom_sort))
+  }
+
+  diyar::episode_group(df, sn=sn, date = "dts", strata= "sr", case_length = "epl", episode_type = "fixed", episodes_max = episodes_max,
+                       bi_direction = bi_direction , data_source = !!ds, custom_sort = "user_srt",
+                       from_last = from_last, overlap_method = overlap_method,
+                       display = display, episode_unit = episode_unit, group_stats = group_stats, deduplicate = deduplicate,to_s4 = to_s4)
 }
 
 #' @rdname episode_group
 #' @export
-rolling_episodes <- function(x,  strata=NULL, case_length, recurrence_length=NULL, from_last = FALSE, overlap_method = c("across","inbetween","aligns_start","aligns_end","chain"), deduplicate = FALSE, display = TRUE){
+rolling_episodes <- function(date, sn = NULL, strata = NULL, case_length, recurrence_length=NULL, episode_unit = "days", episodes_max = Inf, rolls_max = Inf, data_source = NULL, custom_sort = NULL,
+                           from_last = FALSE, overlap_method = c("across","inbetween","aligns_start","aligns_end","chain"),
+                           bi_direction= FALSE, group_stats = FALSE, display = TRUE, deduplicate = FALSE, x, to_s4 = FALSE){
+
+  if(to_s4 == FALSE){
+    if (is.null(getOption("diyar.rolling_episodes.output"))){
+      options("diyar.rolling_episodes.output"= TRUE)
+    }
+    if (getOption("diyar.rolling_episodes.output")){
+      message(paste("The default output of rolling_episodes() will be changed to epid objects in the next release.",
+                    "Please consider switching earlier by using 'to_s4=TRUE' or to_s4()",
+                    "This message is displayed once per session.", sep = "\n"))
+    }
+    options("diyar.rolling_episodes.output"= FALSE)
+  }
+  if (!missing(x)) {
+    warning("'x' is deprecated; please use 'date' instead."); date <- x
+  }
 
   if(!is.character(overlap_method)) stop(paste("'overlap_method' must be a character object"))
   if(all(!tolower(overlap_method) %in% c("across","chain","aligns_start","aligns_end","inbetween"))) stop(paste("`overlap_method` must be either 'across','chain','aligns_start','aligns_end' or 'inbetween'"))
   if(!(is.logical(from_last) & is.logical(display) & is.logical(deduplicate) )) stop(paste("'from_last', 'deduplicate' and 'display' must be TRUE or FALSE"))
   if(!all(is.finite(case_length) & case_length >= 0) ) stop(paste("'case_length' must be a numeric based object of length 1",sep=""))
   if(!(all(is.finite(recurrence_length) & recurrence_length >= 0) | is.null(recurrence_length)) ) stop(paste("'recurrence_length' must be a numeric based object of length 1",sep=""))
-  if(!all(is.finite(x))) stop(paste("All 'x' values must be a date, datetime, number_line object or a numeric based object",sep=""))
-  if(!(length(case_length) %in% c(1, length(x)))) stop(paste("length of 'case_length' must be 1 or the same as 'x'",sep=""))
-  if(!(length(recurrence_length) %in% c(1, length(x)) | (length(recurrence_length) ==0 & is.null(recurrence_length)))) stop(paste("length of 'recurrence_length' must be 1 or the same as 'x'",sep=""))
-  if(!(length(strata) %in% c(1, length(x)) | (length(strata) ==0 & is.null(strata)))) stop(paste("length of 'strata' must be 1 or the same as 'x'",sep=""))
+  if(!all(is.finite(date))) stop(paste("All 'date' values must be a date, datetime, number_line object or a numeric based object",sep=""))
+  if(!(length(case_length) %in% c(1, length(date)))) stop(paste("length of 'case_length' must be 1 or the same as 'date'",sep=""))
+  if(!(length(recurrence_length) %in% c(1, length(date)) | (length(recurrence_length) ==0 & is.null(recurrence_length)))) stop(paste("length of 'recurrence_length' must be 1 or the same as 'date'",sep=""))
+  if(!(length(strata) %in% c(1, length(date)) | (length(strata) ==0 & is.null(strata)))) stop(paste("length of 'strata' must be 1 or the same as 'date'",sep=""))
+  if(!(length(data_source) %in% c(1, length(date)) | (length(data_source) ==0 & is.null(data_source)))) stop(paste("length of 'data_source' must be 1 or the same as 'date'",sep=""))
+  if(!(length(custom_sort) %in% c(1, length(date)) | (length(custom_sort) ==0 & is.null(custom_sort)))) stop(paste("length of 'custom_sort' must be 1 or the same as 'date'",sep=""))
 
-  fmt <- function(g) formatC(g, format="d", big.mark=",")
-  if(is.null(recurrence_length)){
-    recurrence_length <- case_length
-  }
+  df <- data.frame(dts = date, epl = case_length, stringsAsFactors = FALSE)
 
-  if(any(!diyar::is.number_line(x))){
-    x  <- diyar::as.number_line(x)
-  }
-
-  x@gid <- 1:length(x)
   if(is.null(strata)){
-    x <- sort.number_line(x, decreasing = from_last)
-    s <- rep(1, length(x))
+    df$sr <- 1
   }else{
-    x <- data.frame(id= x@id, s = strata, l = x@start, nl = x)
-    if(from_last) x$l <- -as.numeric(x$l)
-    db <- x[order(x$s, x$l, x$id),]
-    x <- db$nl
-    s <- as.numeric(as.factor(db$s))
-    rm(db)
+    df$sr <- strata
   }
 
-  pr_sn <- diyar::number_line(0,0, gid =x@gid, id =x@id)
-  if(any(duplicated(x@id) | is.na(x@id))) stop(paste("'id' slot of the number_line object must be contain unique finite numbers",sep=""))
-  x@gid <- x@id
-
-  x <- diyar::reverse_number_line(x, "decreasing")
-
-  x@id <- 1:length(x@id)
-
-  j <- 0
-  l <- NULL
-  t <- rep(0, length(x))
-  pt <- ifelse(from_last,"start","end")
-  while (min(t) ==0 & j!=length(x)){
-    total_1 <- length(t[t==0])
-    if(display){cat(paste("Episode window ",j+1,".\n", sep=""))}
-
-    if(length(l)==0 | is.null(l)){
-      l <- x[t==0][1]
-      l_s <- s[t==0][1]
-      e <- case_length
-    }else{
-      e <- recurrence_length
-    }
-
-    h <- (x@id == l@id | (diyar::overlap(diyar::expand_number_line(l, by=e, pt), x, method = overlap_method) & s==l_s))
-    x[which(h)]@.Data <- as.numeric(max(diyar::right_point(x[which(h),]))) - as.numeric(min(x[which(h),]@start))
-    x[which(h)]@start <- min(x[which(h),]@start)
-    x[which(h)]@gid <- l@gid
-
-    if(min(t[which(h)])==1){
-      l <- NULL
-    }else{
-      l <- x[which(h)]
-      l <- l[length(l)]
-
-      l_s <- s[which(h)]
-      l_s <- l_s[length(l_s)]
-    }
-
-    tagged_1 <- length(h[h & t==0])
-    t[which(h)] <- 1
-    if(display){
-      cat(paste(fmt(tagged_1)," of ", fmt(total_1)," record(s) grouped into episodes. ", fmt(total_1-tagged_1)," records not yet grouped.\n", sep =""))
-    }
-    if(min(t)==1) break
-    j <- j + 1
+  if(is.null(sn)){
+    df$sn <- 1:nrow(df)
+  }else{
+    df$sn <- sn
   }
 
-  x@id <- pr_sn@id
-  db <- data.frame(nl=x, sn=pr_sn@gid)
-  db <- db[order(db$sn),]
-  x <- db$nl
-  rm(db)
-  if(deduplicate) x <- unique.number_line(x)
+  if(is.null(data_source)){
+    df$ds <- 1
+    ds <- NULL
+  }else{
+    df$ds <- data_source
+    ds <- "ds"
+  }
 
-  if(from_last) x <- diyar::reverse_number_line(x)
-  return(x)
+  if(is.null(custom_sort)){
+    df$user_srt <- 0
+  }else{
+    df$user_srt <- as.numeric(as.factor(custom_sort))
+  }
+
+  if(is.null(recurrence_length)){
+    df$rc_epl <- df$epl
+  }else{
+    df$rc_epl <- recurrence_length
+  }
+
+  diyar::episode_group(df, sn=sn, date = "dts", strata= "sr", case_length = "epl", episode_type = "rolling", episodes_max = episodes_max,
+                       bi_direction = bi_direction , data_source = !!ds, custom_sort = "user_srt",
+                       from_last = from_last, overlap_method = overlap_method, recurrence_length = "rc_epl", rolls_max = rolls_max,
+                       display = display, episode_unit = episode_unit, group_stats = group_stats, deduplicate = deduplicate, to_s4 = to_s4)
 }
