@@ -17,12 +17,7 @@ data$date_int@id <- 1
 # episode grouping with episode_group()
 test_1 <- episode_group(head(data,10), strata = pid, date = date, case_length = episode_len, group_stats = TRUE)
 
-# episode grouping with fixed episodes()
-test_1$epid_interval_2 <- mutate(head(data,10), epid_interval_2 = fixed_episodes(strata = pid, x = date, case_length = d))[["epid_interval_2"]]
-test_1$epid_interval_3 <- mutate(head(data,10), epid_interval_3 = fixed_episodes(x = date, case_length = d[[1]]))[["epid_interval_3"]]
-
 t_ds <- head(data,10)
-epids_dedup <- fixed_episodes(x = t_ds$date, case_length = t_ds$d[[1]], deduplicate = TRUE)
 
 test_that("test that row positions of the resulting dataframe are the same as supplied", {
   expect_equal(test_1$sn, head(data,10)$rd_id)
@@ -36,19 +31,14 @@ e_int <- c(
   )
 
 test_that("test that test episode identifier is as expected for fixed episodes", {
-  expect_error(mutate(head(data,10), epid_interval_4 = fixed_episodes(x = date_int, case_length = d[[1]])),
-               "'id' slot of the number_line object must be contain unique finite numbers")
   expect_equal(test_1$epid, c(1,1,1,4,4,4,7,7,7,10))
   expect_equal(test_1$case_nm, rep(c("Case",rep("Duplicate",2)),4)[1:10] )
 
   e_int@id <- 1:10
   e_int@gid <- c(1,1,1,4,4,4,7,7,7,10)
   expect_equal(test_1$epid_interval, e_int)
-  expect_equal(test_1$epid_interval, test_1$epid_interval_2)
-  expect_equal(test_1$epid_interval, test_1$epid_interval_3)
   expect_equal(test_1$epid_total, c(rep(3,9),1))
   expect_equal(test_1$epid_length, as.difftime(c(rep(6,9),0), units = "days" ))
-  expect_equal(unique(test_1$epid_interval_3), epids_dedup)
 })
 
 # Test 2 - Case assignment - Reverse chronological order
@@ -60,10 +50,6 @@ cbind(data_2,
       rename_all(episode_group(data_2, strata = pid, date = date, case_length = episode_len_s, display = FALSE, from_last = TRUE, group_stats = TRUE), funs(paste(.,2,sep=".")))
 )
 
-#with fixed_episodes()
-test_2$epid_interval.1.1 <- mutate(data_2, epid_interval.1.1 = fixed_episodes(strata = pid, x = date, case_length = d))[["epid_interval.1.1"]]
-test_2$epid_interval.2.1 <- mutate(data_2, epid_interval.2.1 = fixed_episodes(strata = pid, x = date, case_length = d, from_last = TRUE))[["epid_interval.2.1"]]
-
 e_int.1 <- c(
   rep(number_line(dmy_hms("01/04/2018 00:00:00"), dmy_hms("13/04/2018 00:00:00")), 5),
   rep(number_line(dmy_hms("16/04/2018 00:00:00"), dmy_hms("28/04/2018 00:00:00")), 5)
@@ -92,9 +78,6 @@ test_that("test reverse episode grouping", {
   expect_equal(test_2$epid_total.2, rep(5,10))
   expect_equal(test_2$epid_length.2, as.difftime(rep(-12,10), units = "days" ))
 
-  #test that fixed_episodes() gives the same result as episode_group()
-  expect_equal(test_2$epid_interval.1, test_2$epid_interval.1.1)
-  expect_equal(test_2$epid_interval.2, test_2$epid_interval.2.1)
   })
 
 # Test 3 - Rolling episodes
@@ -102,12 +85,6 @@ test_3 <- cbind(data_2,
       rename_all(episode_group(data_2, sn=rd_id, strata = pid, date = date, case_length = episode_len_s, episode_type ="rolling", display = FALSE, from_last = FALSE, group_stats = TRUE), funs(paste(.,1,sep="."))),
       rename_all(episode_group(data_2, sn=rd_id, strata = pid, date = date, case_length = episode_len_s, episode_type ="rolling", display = FALSE, from_last = TRUE, group_stats = TRUE), funs(paste(.,2,sep=".")))
       )
-test_3$epid_interval.1.1 <- mutate(data_2, epid_interval.1.1 = rolling_episodes(strata = pid, x = date, case_length = d))[["epid_interval.1.1"]]
-test_3$epid_interval.2.1 <- mutate(data_2, epid_interval.2.1 = rolling_episodes(x = date, case_length = d[1], from_last = TRUE))[["epid_interval.2.1"]]
-test_3$epid_interval.2.2 <- mutate(data_2, epid_interval.2.2 = rolling_episodes(x = date, strata=pid, case_length = d[1], from_last = TRUE))[["epid_interval.2.2"]]
-
-t_ds <- data_2
-rolling_epids <- rolling_episodes(strata = t_ds$pid, x = t_ds$date, case_length = t_ds$d, deduplicate = TRUE)
 
 e_int.1 <- c(
   rep(number_line(dmy_hms("01/04/2018 00:00:00"), dmy_hms("28/04/2018 00:00:00")), 10)
@@ -135,14 +112,6 @@ test_that("test rolling/recurring episodes", {
   expect_equal(test_3$epid_total.2, rep(10,10))
   expect_equal(test_3$epid_length.2, as.difftime(rep(-27,10), units = "days" ))
 
-  #test that rolling_episodes() gives the same result as episode_group()
-  expect_equal(test_3$epid_interval.1, test_3$epid_interval.1.1)
-  expect_equal(test_3$epid_interval.2, test_3$epid_interval.2.1)
-  expect_equal(test_3$epid_interval.2, test_3$epid_interval.2.2)
-  expect_equal(unique(test_3$epid_interval.1), rolling_epids)
-
-  expect_error(mutate(data_2, epid_interval.2.2 = rolling_episodes(x = date_int, strata=pid, case_length = d[1])),
-               "'id' slot of the number_line object must be contain unique finite numbers")
 })
 
 # Test 3 - Rolls max
@@ -152,9 +121,6 @@ test_4 <- cbind(data_4,
       rename_all(episode_group(data_4, sn=rd_id, strata = pid, date = date, case_length = episode_len_s, episode_type ="rolling", recurrence_length = recurrence, rolls_max = 1,  display = FALSE, group_stats = TRUE), funs(paste(.,2,sep=".")))
       )
 
-#with rolling_episodes()
-test_4$epid_interval.1.1 <- mutate(data_4, epid_interval.1.1 = rolling_episodes(strata = pid, x = date, case_length = d, recurrence_length =r))[["epid_interval.1.1"]]
-
 e_int.1 <- c(
   rep(number_line(dmy_hms("01/04/2018 00:00:00"), dmy_hms("28/04/2018 00:00:00")), 10)
 )
@@ -181,10 +147,6 @@ test_that("test user defined recurrence length and roll_max", {
   expect_equal(test_4$epid_interval.2, e_int.2)
   expect_equal(test_4$epid_total.2, c(rep(6,6), rep(4,4)))
   expect_equal(test_4$epid_length.2, as.difftime(c(rep(15,6), rep(9,4)), units = "days" ))
-
-  #test that rolling_episodes() gives the same result as episode_group()
-  expect_equal(test_4$epid_interval.1, test_4$epid_interval.1.1)
-
 })
 
 # Test 5 - Episodes max
@@ -490,43 +452,6 @@ test_10a.1 <- bind_cols(hospital_infections,
                                       episode_type="rolling", display = FALSE, data_source = infection, group_stats = TRUE)) %>%
   select(-sn)
 
-t_ds <- hospital_infections
-
-t_ds$date <- dmy_hms(format(t_ds$date, "%d/%m/%Y 00:00:00"))
-t_ds$epi_len <- as.numeric(duration(t_ds$epi_len, "days"))
-
-strata_rolling_epids <- rolling_episodes(x=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id)
-test_that("testing; stratified grouping 1.1", {
-  expect_equal(test_10a.1$epid_interval, strata_rolling_epids)
-})
-
-test_that("test fixed and rolling episode funcs errors", {
-  expect_error(rolling_episodes(x=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, overlap_method = "XX"), "`overlap_method` must be either 'across','chain','aligns_start','aligns_end' or 'inbetween'")
-  expect_error(fixed_episodes(x=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, overlap_method = "XX"), "`overlap_method` must be either 'across','chain','aligns_start','aligns_end' or 'inbetween'")
-
-  expect_error(rolling_episodes(x=c(t_ds$date[1:10],NA), case_length = t_ds$epi_len, strata = t_ds$patient_id), "All 'x' values must be a date, datetime, number_line object or a numeric based object")
-  expect_error(fixed_episodes(x=c(t_ds$date[1:10],NA), case_length = t_ds$epi_len, strata = t_ds$patient_id), "All 'x' values must be a date, datetime, number_line object or a numeric based object")
-
-  expect_error(rolling_episodes(x=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, overlap_method = 1), "'overlap_method' must be a character object")
-  expect_error(fixed_episodes(x=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, overlap_method = 2), "'overlap_method' must be a character object")
-  expect_error(rolling_episodes(x=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, from_last = 1), "'from_last', 'deduplicate' and 'display' must be TRUE or FALSE")
-  expect_error(fixed_episodes(x=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, from_last = 1), "'from_last', 'deduplicate' and 'display' must be TRUE or FALSE")
-  expect_error(rolling_episodes(x=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, display = 1), "'from_last', 'deduplicate' and 'display' must be TRUE or FALSE")
-  expect_error(fixed_episodes(x=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, display = 1), "'from_last', 'deduplicate' and 'display' must be TRUE or FALSE")
-  expect_error(rolling_episodes(x=t_ds$date, case_length = -1, strata = t_ds$patient_id), "'case_length' must be a numeric based object of length 1")
-  expect_error(fixed_episodes(x=t_ds$date, case_length = Inf, strata = t_ds$patient_id), "'case_length' must be a numeric based object of length 1")
-  expect_error(rolling_episodes(x=t_ds$date, case_length = c(1,1), strata = t_ds$patient_id), "length of 'case_length' must be 1 or the same as 'x'")
-  expect_error(fixed_episodes(x=t_ds$date, case_length = c(1,1), strata = t_ds$patient_id), "length of 'case_length' must be 1 or the same as 'x'")
-
-  expect_error(rolling_episodes(x=t_ds$date, case_length = t_ds$epi_len, recurrence_length = -1, strata = t_ds$patient_id), "'recurrence_length' must be a numeric based object of length 1")
-  expect_error(rolling_episodes(x=t_ds$date, case_length = t_ds$epi_len, recurrence_length = t_ds$epi_len[1:2],strata = t_ds$patient_id), "length of 'recurrence_length' must be 1 or the same as 'x'")
-  expect_error(rolling_episodes(x=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id[1:2]), "length of 'strata' must be 1 or the same as 'x'")
-  expect_error(fixed_episodes(x=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id[1:2]), "length of 'strata' must be 1 or the same as 'x'")
-
-
-})
-
-
 # Only three 9-day (difference of 8 days) rolling episode per patient and infection.
 hospital_infections$epi_len <- 8
 hospital_infections$recur <- 30
@@ -659,7 +584,7 @@ test_that("test that error and warning messages are returned correctly", {
   expect_error(episode_group(as.list(dft_8), date=admin_period, sn=rd_id,
                              case_length = epi_len, episode_unit = "months", group_stats = TRUE), "A dataframe is required")
   expect_error(episode_group(dft_8, date=admin_periods, sn=rd_id,
-                             case_length = epi_len, episode_unit = "months", group_stats = TRUE), "object 'admin_periods' not found")
+                             case_length = epi_len, episode_unit = "months", group_stats = TRUE), "'admin_periods' not found")
   expect_error(episode_group(dft_8, date=admin_period, sn=rd_id,
                              case_length = epi_len, episode_unit = "months", group_stats = TRUE), "'rd_id' as 'sn' must be > 0")
   expect_error(episode_group(dft_9, date=admin_period, sn=rd_id,
@@ -667,9 +592,9 @@ test_that("test that error and warning messages are returned correctly", {
   expect_error(episode_group(admissions, date=epi_len, sn=rd_id,
                              case_length = epi_len, episode_unit = "months", group_stats = TRUE), "'epi_len' as 'date' must be a date, datetime or number_line object, and not have missing values")
   expect_error(episode_group(admissions, date=admin_period, sn=rd_id,
-                             case_length = epi_len, episode_unit = "months", group_stats = "TRUE"), "'group_stats', 'from_last' and 'display' must be TRUE or FALSE")
+                             case_length = epi_len, episode_unit = "months", group_stats = "TRUE"), "'group_stats', 'from_last', 'display' and 'to_s4' must be TRUE or FALSE")
   expect_error(episode_group(admissions, date=admin_period, sn=rd_id,
-                             case_length = epi_len, episode_unit = "months", from_last = "TRUE"), "'group_stats', 'from_last' and 'display' must be TRUE or FALSE")
+                             case_length = epi_len, episode_unit = "months", from_last = "TRUE"), "'group_stats', 'from_last', 'display' and 'to_s4' must be TRUE or FALSE")
   expect_error(episode_group(admissions, date=admin_period, sn=rd_id,
                              case_length = epi_len, episode_unit = 1), "'episode_unit' must be a character of length 1")
   expect_error(episode_group(admissions, date=admin_period, sn=rd_id,
@@ -689,3 +614,93 @@ test_that("test that error and warning messages are returned correctly", {
   expect_error(episode_group(dft_11, date=admin_period, sn=rd_id,
                              case_length = epi_len, recurrence_length = recur, episode_type = "rolling"), "'recur' as 'recurrence_length' must be -1 or a positive integer, numeric or double data type")
 })
+
+t_ds <- hospital_infections
+
+t_ds$date <- dmy_hms(format(t_ds$date, "%d/%m/%Y 00:00:00"))
+t_ds$epi_len <- as.numeric(duration(t_ds$epi_len, "days"))
+
+test_that("test fixed and rolling episode funcs errors", {
+  expect_error(rolling_episodes(date=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, overlap_method = "XX"), "`overlap_method` must be either 'across','chain','aligns_start','aligns_end' or 'inbetween'")
+  expect_error(fixed_episodes(date=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, overlap_method = "XX"), "`overlap_method` must be either 'across','chain','aligns_start','aligns_end' or 'inbetween'")
+
+  expect_error(rolling_episodes(date=c(t_ds$date[1:10],NA), case_length = t_ds$epi_len, strata = t_ds$patient_id), "All 'date' values must be a date, datetime, number_line object or a numeric based object")
+  expect_error(fixed_episodes(date=c(t_ds$date[1:10],NA), case_length = t_ds$epi_len, strata = t_ds$patient_id), "All 'date' values must be a date, datetime, number_line object or a numeric based object")
+
+  expect_error(rolling_episodes(date=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, overlap_method = 1), "'overlap_method' must be a character object")
+  expect_error(fixed_episodes(date=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, overlap_method = 2), "'overlap_method' must be a character object")
+  expect_error(rolling_episodes(date=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, from_last = 1), "'from_last', 'deduplicate' and 'display' must be TRUE or FALSE")
+  expect_error(fixed_episodes(date=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, from_last = 1), "'from_last', 'deduplicate' and 'display' must be TRUE or FALSE")
+  expect_error(rolling_episodes(date=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, display = 1), "'from_last', 'deduplicate' and 'display' must be TRUE or FALSE")
+  expect_error(fixed_episodes(date=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id, display = 1), "'from_last', 'deduplicate' and 'display' must be TRUE or FALSE")
+  expect_error(rolling_episodes(date=t_ds$date, case_length = -1, strata = t_ds$patient_id), "'case_length' must be a numeric based object of length 1")
+  expect_error(fixed_episodes(date=t_ds$date, case_length = Inf, strata = t_ds$patient_id), "'case_length' must be a numeric based object of length 1")
+  expect_error(rolling_episodes(date=t_ds$date, case_length = c(1,1), strata = t_ds$patient_id), "length of 'case_length' must be 1 or the same as 'date'")
+  expect_error(fixed_episodes(date=t_ds$date, case_length = c(1,1), strata = t_ds$patient_id), "length of 'case_length' must be 1 or the same as 'date'")
+
+  expect_error(rolling_episodes(date=t_ds$date, case_length = 30, data_source = c(1,1), strata = t_ds$patient_id), "length of 'data_source' must be 1 or the same as 'date'")
+  expect_error(fixed_episodes(date=t_ds$date, case_length = 30, data_source = c(1,1), strata = t_ds$patient_id), "length of 'data_source' must be 1 or the same as 'date'")
+  expect_error(rolling_episodes(date=t_ds$date, case_length = 30, custom_sort = c(1,1), strata = t_ds$patient_id), "length of 'custom_sort' must be 1 or the same as 'date'")
+  expect_error(fixed_episodes(date=t_ds$date, case_length = 30, custom_sort = c(1,1), strata = t_ds$patient_id), "length of 'custom_sort' must be 1 or the same as 'date'")
+
+  expect_error(rolling_episodes(date=t_ds$date, case_length = t_ds$epi_len, recurrence_length = -1, strata = t_ds$patient_id), "'recurrence_length' must be a numeric based object of length 1")
+  expect_error(rolling_episodes(date=t_ds$date, case_length = t_ds$epi_len, recurrence_length = t_ds$epi_len[1:2],strata = t_ds$patient_id), "length of 'recurrence_length' must be 1 or the same as 'date'")
+  expect_error(rolling_episodes(date=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id[1:2]), "length of 'strata' must be 1 or the same as 'date'")
+  expect_error(fixed_episodes(date=t_ds$date, case_length = t_ds$epi_len, strata = t_ds$patient_id[1:2]), "length of 'strata' must be 1 or the same as 'date'")
+
+  expect_warning(fixed_episodes(x=t_ds$date, case_length = t_ds$epi_len), "'x' is deprecated; please use 'date' instead.")
+  expect_warning(rolling_episodes(x=t_ds$date, case_length = t_ds$epi_len), "'x' is deprecated; please use 'date' instead.")
+
+})
+
+
+data("infections")
+
+fixed_epids_1 <- episode_group(infections, case_length = epi_len, date=date, to_s4=TRUE)
+fixed_epids_2 <- fixed_episodes(date = infections$date, case_length = 15, to_s4=TRUE)
+
+fixed_epids_3 <- episode_group(infections, case_length = epi_len, date=date, data_source = infection, to_s4=TRUE)
+fixed_epids_4 <- fixed_episodes(date = infections$date, case_length = 15, data_source = infections$infection, to_s4=TRUE)
+
+fixed_epids_5 <- episode_group(infections, case_length = epi_len, date=date, strata = infection, to_s4=TRUE)
+fixed_epids_6 <- fixed_episodes(date = infections$date, case_length = 15, strata = infections$infection, to_s4=TRUE)
+
+fixed_epids_7 <- episode_group(infections, case_length = epi_len, date=date, custom_sort = infection, to_s4=TRUE)
+fixed_epids_8 <- fixed_episodes(date = infections$date, case_length = 15, custom_sort = infections$infection, to_s4=TRUE)
+
+
+rolling_epids_1 <- episode_group(infections, case_length = epi_len, date=date, episode_type = "rolling", to_s4=TRUE)
+rolling_epids_2 <- rolling_episodes(date = infections$date, case_length = 15, to_s4=TRUE)
+
+rolling_epids_3 <- episode_group(infections, case_length = epi_len, date=date, data_source = infection, episode_type = "rolling", to_s4=TRUE)
+rolling_epids_4 <- rolling_episodes(date = infections$date, case_length = 15, data_source = infections$infection, to_s4=TRUE)
+
+rolling_epids_5 <- episode_group(infections, case_length = epi_len, date=date, strata = infection, episode_type = "rolling", to_s4=TRUE)
+rolling_epids_6 <- rolling_episodes(date = infections$date, case_length = 15, strata = infections$infection, to_s4=TRUE)
+
+rolling_epids_7 <- episode_group(infections, case_length = epi_len, date=date, custom_sort = infection, episode_type = "rolling", to_s4=TRUE)
+rolling_epids_8 <- rolling_episodes(date = infections$date, case_length = 15, custom_sort = infections$infection, to_s4=TRUE)
+rolling_epids_8b <- rolling_episodes(date = infections$date, case_length = 15, recurrence_length = 15, custom_sort = infections$infection, to_s4=TRUE)
+
+dup_f_epid <- episode_group(infections, case_length = epi_len, date=date, to_s4=TRUE, deduplicate = TRUE)
+
+test_that("test fixed and rolling episode values", {
+  expect_equal(fixed_epids_1, fixed_epids_2)
+  expect_equal(fixed_epids_3, fixed_epids_4)
+  expect_equal(fixed_epids_5, fixed_epids_6)
+  expect_equal(fixed_epids_7, fixed_epids_8)
+  expect_equal(rolling_epids_1, rolling_epids_2)
+  expect_equal(rolling_epids_3, rolling_epids_4)
+  expect_equal(rolling_epids_5, rolling_epids_6)
+  expect_equal(rolling_epids_7, rolling_epids_8)
+  expect_equal(rolling_epids_8, rolling_epids_8b)
+  expect_equal(dup_f_epid@.Data,  unique(fixed_epids_1)@.Data)
+
+})
+
+
+test_that("test epid methods", {
+  expect_equal(show(dup_f_epid), c("E-01 (C)","E-04 (C)","E-07 (C)","E-10 (C)"))
+  expect_equal(rep(fixed_epids_1,2), rep(fixed_epids_1,2))
+})
+
